// GPanelSizeEx5.java
// Fullscreen, otherwise normal window

/*
 * This is a Sierpinski fractal.
 * With probability 1/8 divide the current (x, y) value by 3
 * and add either 1/3 or 2/3 to x, or to y, or to both.
 * In order to create the Sierpinski carpet holes,
 * give zero probability to
 * xnew = x/3 + 1.0/3;
 * ynew = y/3 + 1.0/3;
 */

import ch.aplu.util.*;
import java.awt.Color;

class GPanelSizeEx5 extends GPanel
{
  final int MAXITERATIONS = 400000;

  GPanelSizeEx5()
  {
    super(FULLSCREEN, "Sierpinski fractal", null, 0, 420, 0, 420, true);
    color(Color.red);
    drawCarpet();
    text(190, 210, "Enjoy the carpet");
  }

  void drawCarpet()
  {
    double x = 1, y = 1, xnew = 1, ynew = 1;
    double hazard, xplot, yplot;

    for (int iter = 0; iter <= MAXITERATIONS; iter++)
    {
      hazard = 8 * Math.random();
      if (hazard < 1)
      {
        xnew = x / 3;
        ynew = y / 3;
      }
      else
      {
        if (hazard < 2)
        {
          xnew = x / 3 + 2.0 / 3;
          ynew = y / 3 + 2.0 / 3;
        }
        else
        {
          if (hazard < 3)
          {
            xnew = x / 3 + 2.0 / 3;
            ynew = y / 3 + 1.0 / 3;
          }
          else
          {
            if (hazard < 4)
            {
              xnew = x / 3 + 1.0 / 3;
              ynew = y / 3 + 2.0 / 3;
            }
            else
            {
              if (hazard < 5)
              {
                xnew = x / 3 + 1.0 / 3;
                ynew = y / 3;
              }
              else
              {
                if (hazard < 6)
                {
                  xnew = x / 3;
                  ynew = y / 3 + 1.0 / 3;
                }
                else
                {
                  if (hazard < 7)
                  {
                    xnew = x / 3 + 2.0 / 3;
                    ynew = y / 3;
                  }
                  else
                  {
                    xnew = x / 3;
                    ynew = y / 3 + 2.0 / 3;
                  }
                }
              }
            }
          }
        }
      }
      xplot = 10 + 400 * xnew;
      yplot = 10 + 400 * ynew;
      point(xplot, yplot);
      x = xnew;
      y = ynew;
    }
    color(Color.black);
    lineWidth(2);
    pos(210, 210);
    rectangle(400, 400);
  }

  public static void main(String[] args)
  {
    new GPanelSizeEx5();
  }
}
