// MenuEx1.java

import javax.swing.*;
import java.awt.event.*;
import java.awt.Color;
import ch.aplu.util.*;

public class MenuEx1 implements ActionListener
{
  interface State
  {
    int IDLE = 0;
    int PLAYING = 1;
    int EXITING = 2;
  }

  private GPanel p = new GPanel(setMenu(), 0, 8, 0, 8);
  private JMenuItem playItem, stopItem, exitItem, helpItem,
                    aboutItem;
  private int state = State.IDLE;

  public MenuEx1()
  {
    int xold = 0, yold = 0;
    int x, y;
    p.enableRepaint(false);
    drawBoard(xold, yold);
    while (state != State.EXITING)
    {
      Thread.currentThread().yield(); // Pass on to anyone
      while (state == State.PLAYING)
      {
        x = (int)(8 * Math.random());
        y = (int)(8 * Math.random());
        drawBoard(x, y);
        xold = x;
        yold = y;
        Console.delay(500);
      }
    }
    System.exit(0);
  }

  private JMenuBar setMenu()
  {
    JMenu fileMenu = new JMenu("File");
    playItem = new JMenuItem("Play");
    fileMenu.add(playItem);
    playItem.addActionListener(this);
    stopItem = new JMenuItem("Stop");
    fileMenu.add(stopItem);
    stopItem.addActionListener(this);
    exitItem = new JMenuItem("Exit");
    fileMenu.add(exitItem);
    exitItem.addActionListener(this);

    JMenu helpMenu = new JMenu("Help");
    helpItem = new JMenuItem("Help Topics");
    helpMenu.add(helpItem);
    helpItem.addActionListener(this);
    aboutItem = new JMenuItem("About...");
    helpMenu.add(aboutItem);
    aboutItem.addActionListener(this);

    JMenuBar menuBar = new JMenuBar();
    menuBar.add(fileMenu);
    menuBar.add(helpMenu);

    fileMenu.setMnemonic('F');
    helpMenu.setMnemonic('H');

    playItem.setAccelerator(KeyStroke.getKeyStroke('P',
            InputEvent.ALT_MASK));
    stopItem.setAccelerator(KeyStroke.getKeyStroke('S',
            InputEvent.ALT_MASK));
    exitItem.setAccelerator(KeyStroke.getKeyStroke('X',
            InputEvent.ALT_MASK));
    helpItem.setAccelerator(KeyStroke.getKeyStroke('H',
            InputEvent.ALT_MASK));
    aboutItem.setAccelerator(KeyStroke.getKeyStroke('T',
            InputEvent.ALT_MASK));

    return menuBar;
  }

  public void actionPerformed(ActionEvent e)
  {
    if (e.getSource() == playItem)
      state = State.PLAYING;
    if (e.getSource() == stopItem)
      state = State.IDLE;
    if (e.getSource() == exitItem)
      state = State.EXITING;
    if (e.getSource() == helpItem)
      JOptionPane.showMessageDialog(
        null,
        "Select File | Play to (re)start the game\n" +
        "Select File | Stop to stop the game");
    if (e.getSource() == aboutItem)
      JOptionPane.showMessageDialog(
        null,
        " Application written by\n  Aegidius Plss\n" +
        " Berne, Switzerland\n" + " http://www.aplu.ch");
  }

  private void drawBoard(double x, double y)
  {
    p.clear();
    for (int i = 0; i < 8; i++)
      for (int j = 0; j < 8; j++)
        if ((i + j) % 2 == 0)
          p.fillRectangle(i, j, i + 1, j + 1);
    p.image("jman.gif", x, y);
    p.repaint();
  }

  public static void main(String[] args)
  {
    new MenuEx1();
  }
}
