// TimeEx2.java
// For pre J2SE 1.5 versions:
// Because of HiResAlarmTimer, Java3D must be installed

import ch.aplu.util.*;
import java.awt.Color;

public class TimeEx2 implements TimerListener
{
  // Constants
  final double dt = 0.1; // Time interval in s
  final double g = 9.81; // Gravitational accelaration in m/s^2

  // Initial conditions
  final double x0 = 0;
  final double y0 = 0;
  final double v0 = 20; // m/s

  // Variables
  double t = 0;
  double y = y0;
  double v = v0;
  double ymax = 0;
  double tmax = 0;

  // Class instances
  GPanel p = new GPanel("Senkrechter Wurf, Anfangsgeschw. " +
                        v0 + " m/s", -12, 12, -1, 23);
  HiResAlarmTimer swatch =
    new HiResAlarmTimer((long)(1000000 * dt));

  // Constructor
  TimeEx2()
  {
    p.move(-5, 0);
    p.text("y = 0 m");
    p.draw(5, 0);
    p.move(-5, 20);
    p.text("y = 20 m");
    p.draw(5, 20);
    p.color(Color.green);
    swatch.addTimerListener(this);
    swatch.start();
  }

  // Callback method
  public boolean timeElapsed()
  {
    p.move(x0, y);
    p.fillCircle(0.3);

    v = v - g * dt; // New velocity
    y = y + v * dt; // New moveition
    t += dt; // New time

    if (y > ymax) // Move upwards
    {
      ymax = y;
      tmax = t;
    }
    else // Move downwards
      p.color(Color.red);

    if (y >= 0)
      return true; // Rearm timer

    // Terminate simulation
    p.color(Color.black);
    p.move(0, 20);
    p.text("  Flugzeit " + tmax + " s");
    p.move(0, 19);
    p.text("   Wurfhhe: " + ymax + " m");
    p.move(0, 18);
    p.text("   Theorie: " + v0 * v0 / 2 / g + " m");
    return false;
  }

  // Main method
  public static void main(String[] args)
  {
    new TimeEx2();
  }
}
