// Julia.java

import ch.aplu.util.*;
import java.awt.Color;
import ORG.netlib.math.complex.Complex;

class Julia
{
  private int maxIterations = 100;
  private double maxNorm = 50;
  private double step = 0.005;
  private double range = 2;
  private GPanel p = new GPanel(-range, range, -range, range);

  public Julia()
  {
    drawJulia();
  }

  private void drawJulia()
  {
    Complex c = new Complex(-0.5, -0.5);
    Complex z0 = new Complex(-range, -range);
    int it;

    while (z0.im() < range) // outer loop in imag direction
    {
      z0 = new Complex(-range, z0.im() + step);
      while (z0.re() < range) // inner loop in real direction
      {
        z0 = z0.add(new Complex(step));

        // Julia iteration
        Complex z = z0;
        it = 0;
        while (z.norm() < maxNorm && it < maxIterations)
        {
          z = z.mul(z).add(c); // z = z^2 + c
          it++;
        }
        if (it < 3)
          putPixel(z0, Color.darkGray);
        else
          if (it < 5)
            putPixel(z0, Color.orange);
          else
            if (it < 8)
              putPixel(z0, Color.red);
            else
              if (it < 12)
                putPixel(z0, Color.blue);
              else
                if (it < 100)
                  putPixel(z0, Color.yellow);
                else
                  putPixel(z0, Color.black);
      }
    }
  }

  void putPixel(Complex z, Color c)
  {
    p.color(c);
    p.point(z.re(), z.im());
  }

  public static void main(String[] args)
  {
    new Julia();
  }
}
