// MatrixEx3.java
// Save and restore matrices in file
// Jama.matrix is serializable

import Jama.*;
import ch.aplu.util.Console;
import java.io.*;

public class MatrixEx3 extends Console
{
  public MatrixEx3()
  {
    double[][] vals = {
        {1.0, 2.0, 3.0},
        {1.0, 3.0, 5.0},
        {1.0, 4.0, 6.0} };

    Matrix A = new Matrix(vals);
    Matrix Ainv = A.inverse();
    println("Matrix A = ");
    A.print(12, 2);
    println("Matrix Ainv = ");
    Ainv.print(12, 2);

    // Save matrices
    println("Press any key to save matrices...");
    getKeyWait();
    ObjectOutputStream out = null;

    try
    {
      File file = new File("matrix.dat");

      out = new ObjectOutputStream(new FileOutputStream(file));
      out.writeObject(A);
      out.writeObject(Ainv);
      println("File " + file.getPath() + " successfully created.");
    }
    catch (Exception ex)
    {
      println("Fatal error. Exception while writing file");
      println("Press any key to quit...");
      getKeyWait();
      System.exit(1);
    }
    finally
    {
      try
      {
        if (out != null)
          out.close();
      }
      catch (IOException ex) {}
    }

    // Restore matrices
    println("Press any key to restore matrices and perform operation...");
    getKeyWait();
    ObjectInputStream in = null;
    try
    {
      File file = new File("matrix.dat");
      in = new ObjectInputStream(new FileInputStream(file));
      Matrix X = (Matrix)in.readObject();
      Matrix Y = (Matrix)in.readObject();
      Matrix Z = X.times(Y);
      println("Matrix X = ");
      X.print(12, 2);
      println("Matrix Y = ");
      Y.print(12, 2);
      println("Matrix Z = X*Y = ");
      Z.print(12, 2);
      println("Press any key to quit...");
      getKeyWait();
      System.exit(0);
    }
    catch (Exception ex)
    {
      println("Fatal error. Exception while reading file");
      println("Press any key to quit...");
      getKeyWait();
      System.exit(1);
    }
    finally
    {
      try
      {
        if (in != null)
          in.close();
      }
      catch (IOException ex) {}
    }
  }

  public static void main(String[] args)
  {
    new MatrixEx3();
  }
}