// ChatClient.java

import java.awt.EventQueue;
import java.net.*;
import java.io.*;

public class ChatClient
{
  class InputThread extends Thread
  {
    public void run()
    {
      try
      {
        while (true)
          line = netIn.readLine(); // Blocking
      }
      catch (IOException ex)
      {
        isConnected = false;
      }
    }
  }

  private BufferedReader netIn = null;
  private PrintWriter netOut = null;
  private Socket socket = null;
  private String line = null;
  private boolean isConnected = false;
  private ChatPanel p = null;

  public ChatClient(final String hostname, final int port)
  {
    try
    {
      socket = new Socket(hostname, port);
      socket.setSoTimeout(0);

      netIn = new BufferedReader(
        new InputStreamReader(socket.getInputStream()));
      netOut = new PrintWriter(
        new OutputStreamWriter(socket.getOutputStream()));

      EventQueue.invokeAndWait(new Runnable()
      {
        public void run()
        {
          p = new ChatPanel("Simple Chat (Client)");
          p.setOutput(netOut);
          p.appendLine("Connection to '" + hostname +
                       "' on port " + port + " established");
        }
      });

      new InputThread().start();
      isConnected = true;
      p.enableEntry(true);

      while (isConnected)
      {
        try
        {
          Thread.currentThread().sleep(1);
        }
        catch (InterruptedException ex) {}
        if (line != null)
        {
          p.appendLine(line);
          line = null;
        }
      }
    }
    catch (Exception ex) {}
    finally
    {
      try
      {
        if (netIn != null)
          netIn.close();
        if (netOut != null)
          netOut.close();
        if (socket != null)
          socket.close();
      }
      catch (IOException ex) {}
    }
  }

  void dispose()
  {
    if (p != null)
      p.dispose();
  }

  public static void main(String args[])
  {
    String hostname = "localhost";
    int port = 3000;
    if (args.length > 0)
      hostname = args[0];
    if (args.length == 2)
      port = Integer.parseInt(args[1]);
    new ChatClient(hostname, port);
    System.out.println("Connection to '" + hostname +
                       "' on port " + port + " failed");
    System.exit(0);
  }
}
