// DaytimeThreadedServer.java

import java.net.*;
import java.io.*;
import java.util.*;
import ch.aplu.util.Console;

class OutputThread extends Thread
{
  private OutputStream os;
  private boolean isLazy;

  OutputThread(OutputStream os, boolean isLazy)
  {
    this.os = os;
    this.isLazy = isLazy;
  }

  public void run()
  {
    PrintWriter pr =
      new PrintWriter(new OutputStreamWriter(os));

    if (isLazy)
      Console.delay(10000); // lazy server
    Date now = new Date();
    try
    {
      InetAddress address = InetAddress.getLocalHost();
      pr.println("The current daytime at " + address +
                 " is:");
      pr.println(now);
      pr.println("Goodbye");
      pr.flush();
      pr.close();
    }
    catch (Exception ex)
    {
      System.out.println("Error " + ex);

    }
  }
}

public class DaytimeThreadedServer
{
  private static final int SERVICE_PORT = 13;
  private boolean isLazy = true;

  public DaytimeThreadedServer()
  {
    try
    {
      ServerSocket server = new ServerSocket(SERVICE_PORT);
      if (isLazy)
        System.out.print("Lazy ");
      else
        System.out.print("Quick ");
      System.out.println("daytime service started(threaded)");

      while (true)
      {
        Socket client = server.accept();
        System.out.println("Received request on " +
                           new Date() + " from " +
                           client.getInetAddress() + ":" +
                           client.getPort());
        Thread output =
          new OutputThread(client.getOutputStream(), isLazy);
        output.start();
      }
    }
    catch (BindException ex)
    {
      System.err.println("Service already running on port " +
                         SERVICE_PORT);
    }
    catch (IOException ex)
    {
      System.err.println("Error " + ex);
    }
  }

  public static void main(String args[])
  {
    Console.init();
    new DaytimeThreadedServer();
  }
}
