// Metex.java

import java.io.*;
import javax.comm.*;

public class Metex
{
  // ------------ Inner class AcquisitionThread ----------
  private class AcquisitionThread extends Thread
  {
    private final char SYNCHBYTE = 0; // May be any byte
    public void run()
    {
      try
      {
        os.write(SYNCHBYTE);
        data = br.readLine();
      }
      catch (Exception ex) {}
    }
  }

  // ------------ End of inner class ---------------------

  private BufferedReader br;
  private OutputStream os;
  private String data = "";

  public Metex(String portName) throws MetexException
  {
    int baudrate = 1200;
    int databits = SerialPort.DATABITS_7;
    int stopbits = SerialPort.STOPBITS_2;
    int parity = SerialPort.PARITY_NONE;
    char flowControl = 'n';

    SerialPort port = PortCreator.openPort(portName,
                                           baudrate,
                                           databits,
                                           stopbits, parity,
                                           flowControl);
    if (port == null)
    {
      throw new MetexPortException("Unable to open comport " +
                                   portName);
    }
    try
    {
      br = new BufferedReader(
        new InputStreamReader(port.getInputStream()));
      os = port.getOutputStream();
    }
    catch (IOException ex) {}

    // The following two lines are mandatory to
    // activate the serial port in Metex instruments
    port.setDTR(true);
    port.setRTS(false);
  }

  public void close()
  {
    try
    {
      br.close();
      os.close();
    }
    catch (IOException ex) {}
  }

  public String getSample() throws MetexTimeoutException
  {
    int timeout = 700; //  ms
    int retryCount = 3;
    while (true)
    {
      AcquisitionThread at = new AcquisitionThread();
      at.start();
      try
      {
        at.join(timeout); // Wait for the thread to die
      }
      catch (InterruptedException ex) {}

      if (at.isAlive()) // Error, no response
      {
        at.interrupt(); // Stop it by brute force
        retryCount--;
        if (retryCount == 0)
          throw new MetexTimeoutException(
            "Timeout while waiting for data");
      }
      else
        return data;
    }
  }
}
