// VirtualInstrument.java

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.sound.sampled.*;

public class VirtualInstrument extends JFrame
        implements ActionListener
{
  private double[] equalTemp =
  {
    261.626, 293.665, 329.628, 349.228, 391.995, 440.000,
    493.883, 523.252
  };
  private double[] pure =
  {
    264, 297, 330, 352, 396, 440, 495, 528
  };
  private int nbTones = 16;
  private FunctionPlayer[] fPlayers =
    new FunctionPlayer[nbTones];
  private final JButton[] buttons =
  {
    new JButton("C"), new JButton("D"), new JButton("E"),
    new JButton("F"), new JButton("G"), new JButton("A"),
    new JButton("H"), new JButton("C"), new JButton("C"),
    new JButton("D"), new JButton("E"), new JButton("F"),
    new JButton("G"), new JButton("A"), new JButton("H"),
    new JButton("C")
  };
  private final int nbButtons = buttons.length;
  private JLabel[] labels = {new JLabel(
                              "Wohltemperierte Stimmung"),
                             new JLabel("Reine Stimmung")};
  private JPanel[] panels = new JPanel[4];

  public VirtualInstrument()
  {
    for (int i = 0; i < nbTones; i++)
    {
      fPlayers[i] =
        new FunctionPlayer(MyAudioFormat.getAudioFormat());
      if (i < 8)
        fPlayers[i].initData(2, 6, equalTemp[i], 0.1);
      else
        fPlayers[i].initData(2, 6, pure[i - 8], 0.1);
    }

    for (int i = 0; i < 4; i++)
    {
      panels[i] = new JPanel();
      getContentPane().add(panels[i]);
    }
    panels[1].add(labels[0]);
    panels[3].add(labels[1]);
    for (int i = 0; i < nbButtons; i++)
    {
      buttons[i].addActionListener(this);
      if (i < 8)
        panels[0].add(buttons[i]);
      else
        panels[2].add(buttons[i]);
    }

    getContentPane().setLayout(
      new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setTitle("Virtuelles Musikinstrument");
    setSize(450, 160);
    setLocation(200, 100);
    setVisible(true);
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    for (int i = 0; i < nbButtons; i++)
    {
      if (source == buttons[i])
      {
        try
        {
          if (!fPlayers[i].isPlaying()) // inhibit same tone
            fPlayers[i].start(false);
        }
        catch (LineUnavailableException ex)
        {
          System.out.println("Sound card not available");
          System.exit(1);
        }
      }
    }
  }

  public static void main(String args[])
  {
    EventQueue.invokeLater(new Runnable()
    {
      public void run()
      {
        new VirtualInstrument();
      }
    });
  }
}
