// SoundRecorder.java

import javax.sound.sampled.*;
import java.io.*;

public class SoundRecorder
{
  private boolean stopCapture = false;

  private class RecorderThread extends Thread
  {
    public void run()
    {
      byte buf[] = new byte[10000];
      stopCapture = false;
      try
      {
        while (!stopCapture)
        {
          int cnt = targetDataLine.read(buf, 0, buf.length);
          if (cnt > 0)
            data.write(buf, 0, cnt);
        }
        data.close();
      }
      catch (IOException ex)
      {
        System.out.println(ex);
        System.exit(1);
      }
    }
  }

  protected ByteArrayOutputStream data;
  protected AudioFormat audioFormat;
  protected TargetDataLine targetDataLine;

  public SoundRecorder(AudioFormat audioFormat)
  {
    this.audioFormat = audioFormat;
  }

  public void capture(ByteArrayOutputStream data)
          throws LineUnavailableException
  {
    this.data = data;
    DataLine.Info dataLineInfo =
      new DataLine.Info(TargetDataLine.class, audioFormat);
    targetDataLine =
      (TargetDataLine)AudioSystem.getLine(dataLineInfo);
    targetDataLine.open(audioFormat);
    targetDataLine.start();
    new RecorderThread().start();
  }

  public void setStopCapture(boolean b)
  {
    stopCapture = b;
  }

  void writeWavFile(ByteArrayOutputStream data, File file)
  {
    byte[] audioData = data.toByteArray();
    InputStream byteArrayInputStream =
      new ByteArrayInputStream(audioData);
    AudioInputStream audioInputStream =
      new AudioInputStream(byteArrayInputStream, audioFormat,
                           audioData.length /
                           audioFormat.getFrameSize());
    try
    {
      AudioSystem.write(audioInputStream,
                        AudioFileFormat.Type.WAVE, file);
    }
    catch (IOException ex)
    {
      System.out.println(ex);
    }
  }
}
