// StreamingPlayer.java

import javax.sound.sampled.*;
import java.io.*;

abstract class StreamingPlayer
{
  private class PlayerThread extends Thread
  {
    public void run()
    {
      byte buf[] = new byte[10000];
      try
      {
        int cnt;
        while ((cnt = audioInputStream.read(
                buf, 0, buf.length)) != -1)
        {
          if (cnt > 0)
            sourceDataLine.write(buf, 0, cnt);
        }
        sourceDataLine.drain();
        sourceDataLine.close();
      }
      catch (IOException ex)
      {
        System.out.println(ex);
        System.exit(1);
      }
    }
  }

  protected AudioFormat audioFormat;
  protected AudioInputStream audioInputStream;
  private SourceDataLine sourceDataLine;
  private PlayerThread playerThread;

  public void start(boolean doFinish)
          throws LineUnavailableException
  {
    DataLine.Info dataLineInfo =
      new DataLine.Info(SourceDataLine.class, audioFormat);
    sourceDataLine =
      (SourceDataLine)AudioSystem.getLine(dataLineInfo);
    sourceDataLine.open(audioFormat);
    sourceDataLine.start();
    playerThread = new PlayerThread();
    playerThread.start();
    if (doFinish)
      waitToFinish();
  }

  public boolean isPlaying()
  {
    if (playerThread == null)
      return false;
    return (playerThread.isAlive());
  }

  public void waitToFinish()
  {
    if (playerThread == null)
      return;
    try
    {
      playerThread.join();
    }
    catch (InterruptedException ex) {}
  }
}
