// Waveform.java

public interface Waveform
{
  public double f(double t, double freq);

  class F0 implements Waveform

  // Sine
  {
    public double f(double t, double freq)
    {
      return Math.sin(2 * Math.PI * freq * t);
    }
  }

  class F1 implements Waveform

  // Square
  {
    public double f(double t, double freq)
    {
      return square(2 * Math.PI * freq * t);
    }

    private double square(double x)
    {
      x = Math.IEEEremainder(x, 2 * Math.PI);
      return x > 0 ? 1.0 : -1.0;
    }
  }

  class F2 implements Waveform

  // Sawtooth
  {
    public double f(double t, double freq)
    {
      return sawtooth(2 * Math.PI * freq * t);
    }

    private double sawtooth(double x)
    {
      return Math.IEEEremainder(x, 2 * Math.PI);
    }
  }

  class F3 implements Waveform

  // Beat 5 Hz
  {
    public double f(double t, double freq)
    {
      double inc = 5;
      return 0.5 * Math.sin(2 * Math.PI * freq * t) +
             0.5 * Math.sin(2 * Math.PI * (freq + inc) * t);
    }
  }

  class F4 implements Waveform

  // Chirp
  {
    public double f(double t, double freq)
    {
      double upTime = 3;
      double maxFreq = 10 * freq;
      double f = (maxFreq - freq) / upTime * t + freq;
      return Math.sin(2 * Math.PI * f * t);
    }
  }
}
