// BufferedMan.java

import java.awt.*;
import java.awt.image.*;

public class BufferedMan extends Canvas
{
  private Image image;
  private BufferedImage bi;
  private Graphics2D offG2D;
  private int xPos, yPos;

  public BufferedMan(String imagePath, int x, int y)
  {
    image = loadImage(imagePath);
    xPos = x;
    yPos = y;
    setBackground(Color.cyan);
  }

  public Dimension getPreferredSize()
  {
    return new Dimension(400, 400);
  }

  // Inhibit clearing of screen by overriding update()
  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    // Don't call super.paint(g);
    Dimension d = getSize();
    Graphics2D g2D = (Graphics2D)g;

    // Create new offscreen buffer if necessary
    checkOffScreenBuffer();

    // Clear offscreen buffer
    offG2D.setColor(getBackground());
    offG2D.fillRect(0, 0, d.width, d.height);

    // Draw in offscreen buffer
    offG2D.drawImage(image, xPos, yPos, null);

    // Put offscreen buffer on screen
    g.drawImage(bi, 0, 0, null);
  }

  private void checkOffScreenBuffer()
  {
    Dimension d = getSize();
    if (bi == null // no buffer yet

    // Size changed due to resizing
    || bi.getWidth(null) != d.width || bi.getHeight(null)
            != d.height)
    {
      bi = new BufferedImage(d.width, d.height,
                             BufferedImage.TYPE_INT_RGB);
      offG2D = bi.createGraphics();
    }
  }

  public void setPos(int x, int y)
  {
    xPos = x;
    yPos = y;
  }

  public Image loadImage(String imagePath)
  {
    Image image =
      Toolkit.getDefaultToolkit().getImage(imagePath);
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(image, 1);
    try
    {
      mt.waitForAll();
    }
    catch (Exception ex)
    {
      System.out.println("Exception while loading image.");
    }
    return image;
  }
}
