// DbDialog.java

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.sql.*;

class DbDialog extends JFrame implements ActionListener
{
  private DbEx2 app;
  private JPanel contentPane;
  private final int DLG_WIDTH = 350;

  // -------------------- Panes ---------------------------
  private final int NB_PANES = 5;
  private JPanel[] panes = new JPanel[NB_PANES];
  private String[] paneTitles = {"", "Navigation", "Mutation",
                                 "Suche Autor", ""};
  private int[] paneHeights = {100, 70, 70, 60, 20};

  // -------------------- TextFields ----------------------
  private final int NB_TEXTFIELDS = 5;
  private JTextField[] textFields =
    new JTextField[NB_TEXTFIELDS];

  private JTextField searchField = new JTextField(60);

  // -------------------- Labels --------------------------
  private JLabel[] labels = new JLabel[NB_TEXTFIELDS];
  private String[] labelTexts = {"Autor: ", "Titel: ",
                                 "Jahr: ", "Verlag: ",
                                 "Preis: "};

  // -------------------- Buttons -------------------------
  private final static int NB_BUTTONS = 8;
  private JButton[] buttons = new JButton[NB_BUTTONS];
  private String[] buttonTexts = {"First", "Prev", "Next",
                                  "Last", "Delete", "Insert",
                                  "Save", "Search"};

  // -------------------- Status --------------------------
  private JLabel status = new JLabel();

  public DbDialog(String title, DbEx2 app)
  {
    super(title);
    this.app = app;
    setDefaultCloseOperation(
      WindowConstants.DO_NOTHING_ON_CLOSE);
    init();
    pack();
    setResizable(false);
    setLocation(100, 100);
    setVisible(true);
  }

  private void init()
  {
    contentPane = (JPanel)getContentPane();
    contentPane.setLayout(new BoxLayout(contentPane,
                                        BoxLayout.Y_AXIS));
    GridBagLayout gridBag = new GridBagLayout();

    // ----------------- Panes ----------------------------
    for (int i = 0; i < NB_PANES; i++)
    {
      panes[i] = new JPanel();
      contentPane.add(panes[i]);
      panes[i].setPreferredSize(
        new Dimension(DLG_WIDTH, paneHeights[i]));
      if (i != NB_PANES - 1)
      {
        panes[i].setBorder(
          BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder(paneTitles[i]),
            BorderFactory.createEmptyBorder(5, 5, 5, 5)));
      }
    }

    panes[0].setLayout(gridBag);
    panes[0].setBackground(Color.white);
    panes[1].setLayout(new BoxLayout(panes[1],
                                     BoxLayout.X_AXIS));
    panes[2].setLayout(new BoxLayout(panes[2],
                                     BoxLayout.X_AXIS));
    panes[3].setLayout(new BoxLayout(panes[3],
                                     BoxLayout.X_AXIS));
    panes[4].setLayout(new BorderLayout());
    panes[4].setBorder(new BevelBorder(BevelBorder.LOWERED));
    panes[4].add(status);

    // ----------------- TextFields -----------------------
    for (int i = 0; i < NB_TEXTFIELDS; i++)
    {
      textFields[i] = new JTextField(60);
      labels[i] = new JLabel(labelTexts[i]);
    }
    addLabelTextRows(labels, textFields, gridBag, panes[0]);

    panes[3].add(searchField);

    // ----------------- Buttons --------------------------
    for (int i = 0; i < NB_BUTTONS; i++)
    {
      buttons[i] = new JButton(buttonTexts[i]);
      buttons[i].addActionListener(this);
      if (i < 4)
        panes[1].add(buttons[i]);
      else
        if (i < NB_BUTTONS - 1)
          panes[2].add(buttons[i]);
        else
          panes[3].add(buttons[i]);
    }
  }

  public void actionPerformed(ActionEvent evt)
  {
    Object source = evt.getSource();
    for (int i = 0; i < NB_BUTTONS; i++)
      if (source == buttons[i])
        app.doIt(i);
  }

  void setButtonsDisabled()
  {
    switch (app.getState())
    {
      case NAVIGATE :
      case SEARCH :
        for (int i = 0; i < NB_BUTTONS; i++)
          if (i != 6)
            buttons[i].setEnabled(true);
        buttons[0].requestFocus();
        break;

      case INSERT :
        for (int i = 0; i < NB_BUTTONS; i++)
          if (i != 6)
            buttons[i].setEnabled(false);
        buttons[6].requestFocus();
        break;
    }
  }

  void showRecord(ResultSet rs)
  {
    try
    {
      for (int i = 0; i < NB_TEXTFIELDS; i++)
        textFields[i].setText(rs.getString(i + 2));
    }
    catch (SQLException ex)
    {
      System.out.println("Sorry. Got SQLException: " +
                         ex.getMessage());
    }
  }

  void showStatus(String text)
  {
    status.setText(" " + text);
  }

  String[] getFields()
  {
    String[] s = new String[NB_TEXTFIELDS];
    for (int i = 0; i < NB_TEXTFIELDS; i++)
    {
      s[i] = textFields[i].getText();
      s[i] = s[i].trim();
    }
    return s;
  }

  String getSearchString()
  {
    String s = searchField.getText();
    s = s.trim();
    return s;
  }

  void setSearchString(String s)
  {
    searchField.setText(s);
  }

  void clearFields()
  {
    for (int i = 0; i < NB_TEXTFIELDS; i++)
      textFields[i].setText("");
    searchField.setText("");
  }

  // Method from TextSamplerDemo.java
  // Sun, "The JFC Swing Tutorial"
  private void addLabelTextRows(JLabel[] label,
                                JTextField[] textField,
                                GridBagLayout gridbag,
                                Container container)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.anchor = GridBagConstraints.EAST;
    int numLabels = label.length;

    for (int i = 0; i < numLabels; i++)
    {
      c.gridwidth = GridBagConstraints.RELATIVE; //next-to-last
      c.fill = GridBagConstraints.NONE; //reset to default
      c.weightx = 0.0; //reset to default
      gridbag.setConstraints(label[i], c);
      container.add(label[i]);

      c.gridwidth = GridBagConstraints.REMAINDER; //end row
      c.fill = GridBagConstraints.HORIZONTAL;
      c.weightx = 1.0;
      gridbag.setConstraints(textField[i], c);
      container.add(textField[i]);
    }
  }

  public void processWindowEvent(WindowEvent evt)
  {
    super.processWindowEvent(evt);
    if (evt.getID() == WindowEvent.WINDOW_CLOSING)
      app.doIt(8);
  }
}
